{********************************************************************************
            Copyright (c) PilotLogic Software House                  
  
 Package pl_GlassDocking
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)  
                                                                               
   ****** BEGIN LICENSE BLOCK *****     
                                                              
   The contents of this file are used with permission, subject to the Mozilla   
   Public License Version 2.0 (the "License"); you may not use this file except 
   in compliance with the License. You may obtain a copy of the License at      
   https://www.mozilla.org/en-US/MPL/2.0/                                     
                                                                               
   Software distributed under the License is distributed on an "AS IS" basis,   
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for 
   the specific language governing rights and limitations under the License. 
   
   ****** END LICENSE BLOCK *****  
  
**********************************************************************************}

unit gd_dockingengine;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,Controls,Forms,Dialogs,
  LCLIntf,
  gd_dockingbase;

type

TGlassDockEngine = class(TComponent)
 protected
    FActive:Boolean;
    FDockPanel:TGlassDockPanel;
    FOnCreateControl: TGDMCreateControlEvent;
    FStartExecuteDock  : TGDMStartExecuteDock;  
    FEndExecuteDock    : TGDMEndExecuteDock;
    FStartExecuteUnDock: TGDMStartExecuteUnDock;
    FEndExecuteUnDock  : TGDMEndExecuteUnDock;
    function  GetDockMaster:TGlassDockMaster;
    procedure SetActive(const aValue:Boolean);
 public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure MakeDockPanel(APanel:TGlassDockPanel; ResizePolicy: TADMResizePolicy);  

    procedure ScreenAddForm(Sender: TObject; aForm: TCustomForm);
    procedure ShowForm(AForm: TCustomForm; BringToFront: boolean);

    procedure SaveAllToFile(const Filename: string);
    function  LoadAllFromFile(const Filename: string): boolean;

    property DockMaster: TGlassDockMaster read GetDockMaster;
 published
    property Active:boolean read FActive write SetActive;
    property DockPanel:TGlassDockPanel read FDockPanel write FDockPanel;
    property OnCreateControl: TGDMCreateControlEvent read FOnCreateControl write FOnCreateControl;
    property OnStartExecuteDock  :TGDMStartExecuteDock read FStartExecuteDock write FStartExecuteDock;    
    property OnEndExecuteDock  :TGDMEndExecuteDock read FStartExecuteDock write FStartExecuteDock;
    property OnStartExecuteUnDock:TGDMStartExecuteUnDock read FStartExecuteUnDock write FStartExecuteUnDock;
    property OnEndExecuteUnDock:TGDMEndExecuteUnDock read FStartExecuteUnDock write FStartExecuteUnDock;

 end;

implementation

constructor TGlassDockEngine.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TGlassDockEngine.Destroy;
begin
  Screen.RemoveAllHandlersOfObject(Self);
  inherited Destroy;
end;

function  TGlassDockEngine.GetDockMaster:TGlassDockMaster;
begin
  result:=VarBaseDockMaster;
end;

procedure TGlassDockEngine.SetActive(const aValue:Boolean);
begin
  fActive:=aValue;
  if csDesigning in ComponentState then exit;

 if aValue=false then
  begin
    Screen.RemoveAllHandlersOfObject(Self);

     if VarBaseDockMaster=Nil then
      begin
        exit;
      end else
      begin

        VarBaseDockMaster.OnCreateControl:=nil;
        VarBaseDockMaster.OnStartExecuteDock:=nil;
        VarBaseDockMaster.OnStartExecuteUnDock:=nil;
        VarBaseDockMaster.OnEndExecuteDock:=nil;
        VarBaseDockMaster.OnEndExecuteUnDock:=nil;
        VarBaseDockMaster.Free;
      end;

  end else
  begin

     if VarBaseDockMaster=Nil then
      begin
        VarBaseDockMaster:=TGlassDockMaster.Create(self);
        VarBaseDockMaster.LoadDefaultSettings;
        VarBaseDockMaster.OnCreateControl:=FOnCreateControl;
        VarBaseDockMaster.OnStartExecuteDock:=FStartExecuteDock;    
        VarBaseDockMaster.OnEndExecuteDock:=FEndExecuteDock;
        VarBaseDockMaster.OnStartExecuteUnDock:=FStartExecuteUnDock;
        VarBaseDockMaster.OnEndExecuteUnDock:=FEndExecuteUnDock;
      end else
      begin
        exit;
      end;

   Screen.AddHandlerFormVisibleChanged(@ScreenAddForm);
  end;
end;

procedure TGlassDockEngine.ShowForm(AForm: TCustomForm; BringToFront: boolean);
begin
  if AForm = nil then exit;
  if FOnCreateControl<>nil then exit;  //User ShowForm ....
  if (AForm=application.MainForm) and (FDockPanel<>nil) then
   MakeDockPanel(FDockPanel,admrpChild) else
   VarBaseDockMaster.MakeDockable(AForm);
end;

procedure TGlassDockEngine.ScreenAddForm(Sender: TObject; aForm: TCustomForm);
begin
  if DockMaster=nil then exit;
  ShowForm(aForm,false);
end;

procedure TGlassDockEngine.MakeDockPanel(APanel:TGlassDockPanel; ResizePolicy: TADMResizePolicy);
begin
  if DockMaster=nil then exit;
  DockMaster.MakeDockPanel(APanel,ResizePolicy);
end;

procedure TGlassDockEngine.SaveAllToFile(const Filename: string);
begin
  if DockMaster=nil then exit;

  DockMaster.SaveAllToFile(Filename);
end;

function  TGlassDockEngine.LoadAllFromFile(const Filename: string): boolean;
begin
  if DockMaster=nil then exit;
  result:=DockMaster.LoadAllFromFile(Filename);
end;

end.

