{********************************************************************************
            Copyright (c) PilotLogic Software House                  
  
 Package pl_GlassDocking
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)  
                                                                               
   ****** BEGIN LICENSE BLOCK *****     
                                                              
   The contents of this file are used with permission, subject to the Mozilla   
   Public License Version 2.0 (the "License"); you may not use this file except 
   in compliance with the License. You may obtain a copy of the License at      
   https://www.mozilla.org/en-US/MPL/2.0/                                     
                                                                               
   Software distributed under the License is distributed on an "AS IS" basis,   
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for 
   the specific language governing rights and limitations under the License. 
   
   ****** END LICENSE BLOCK *****  
  
**********************************************************************************}

unit gd_dockingoptionsdlg;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ButtonPanel,
  StdCtrls, ComCtrls, LCLProc, ColorBox, Spin, ExtCtrls, gd_dockingbase,
  gd_dockingStr;

type
  TAnchorDockOptionsFlag = (
    adofShow_ShowHeader
    );
  TAnchorDockOptionsFlags = set of TAnchorDockOptionsFlag;

  { TGlassDockOptionsFrame }

  TGlassDockOptionsFrame = class(TFrame)
    ButtonSetDafault: TButton;
    ShowHeaderCloseButtonCheckBox: TCheckBox;
    DockSitesMultiLinePages: TCheckBox;
    HeaderBorderColor: TLabel;
    HeaderFontColor: TLabel;
    HeaderIconsGroup: TLabel;
    HeaderFontColorComboBox: TColorBox;
    HeaderIconsGroupSpin: TSpinEdit;
    ImageClose: TImage;
    ImageMinimaze: TImage;
    PanelForImages: TPanel;
    SplitterColor: TLabel;
    HeaderColorComboBox: TColorBox;
    DragThresholdLabel: TLabel;
    DragThresholdTrackBar: TTrackBar;
    HeaderAlignLeftLabel: TLabel;
    HeaderAlignLeftTrackBar: TTrackBar;
    HeaderAlignTopLabel: TLabel;
    HeaderAlignTopTrackBar: TTrackBar;
    HeaderColor: TLabel;
    HeaderBorderColorComboBox: TColorBox;
    SplitterColorComboBox: TColorBox;
    HideHeaderCaptionForFloatingCheckBox: TCheckBox;
    DockSitesCanBeMinimized: TCheckBox;
    ScaleOnResizeCheckBox: TCheckBox;
    ShowHeaderCaptionCheckBox: TCheckBox;
    ShowHeaderCheckBox: TCheckBox;
    SplitterWidthLabel: TLabel;
    SplitterWidthTrackBar: TTrackBar;
    procedure ButtonSetDafaultClick(Sender: TObject);
    procedure HeaderIconsGroupSpinChange(Sender: TObject);
    procedure OkClick(Sender: TObject);
    procedure DragThresholdTrackBarChange(Sender: TObject);
    procedure HeaderAlignLeftTrackBarChange(Sender: TObject);
    procedure HeaderAlignTopTrackBarChange(Sender: TObject);
    procedure SplitterWidthTrackBarChange(Sender: TObject);
  private
    FFlags: TAnchorDockOptionsFlags;
    FMaster: TGlassDockMaster;
    FSettings: TGlassDockSettings;
    procedure SetFlags(AValue: TAnchorDockOptionsFlags);
    procedure SetMaster(const AValue: TGlassDockMaster);
    procedure SetSettings(AValue: TGlassDockSettings);
    procedure UpdateDragThresholdLabel;
    procedure UpdateHeaderAlignTopLabel;
    procedure UpdateHeaderAlignLeftLabel;
    procedure UpdateSplitterWidthLabel;
    procedure ApplyFlags;
  public
    procedure SaveToMaster;
    procedure LoadFromMaster;
    procedure LoadDefault;
    procedure SaveToSettings(TheSettings: TGlassDockSettings);
    procedure LoadFromSettings(TheSettings: TGlassDockSettings);
    property Master: TGlassDockMaster read FMaster write SetMaster;
    property Settings: TGlassDockSettings read FSettings write SetSettings;
    property Flags: TAnchorDockOptionsFlags read FFlags write SetFlags;
  end;

function ShowGlassDockOptions(ADockMaster: TGlassDockMaster): TModalResult;

implementation

function ShowGlassDockOptions(ADockMaster: TGlassDockMaster): TModalResult;
var
  Dlg: TForm;
  OptsFrame: TGlassDockOptionsFrame;
  BtnPanel: TButtonPanel;
begin
  Dlg := TForm.Create(nil);
  try
    Dlg.DisableAutoSizing;
    try
    Dlg.Position := poScreenCenter;
    Dlg.AutoSize := True;
    Dlg.Caption := adrsGeneralDockingOptions;

    OptsFrame := TGlassDockOptionsFrame.Create(Dlg);
    OptsFrame.Align := alClient;
    OptsFrame.Parent := Dlg;
    OptsFrame.Master := ADockMaster;

    BtnPanel := TButtonPanel.Create(Dlg);
    BtnPanel.ShowButtons := [pbOK, pbCancel];
    BtnPanel.OKButton.OnClick := @OptsFrame.OkClick;
    BtnPanel.Parent := Dlg;
    finally
      Dlg.EnableAutoSizing;
    end;
    Result := Dlg.ShowModal;
  finally
    Dlg.Free;
  end;
end;

{$R *.frm}

procedure TGlassDockOptionsFrame.HeaderAlignLeftTrackBarChange(Sender: TObject);
begin
  UpdateHeaderAlignLeftLabel;
end;

procedure TGlassDockOptionsFrame.HeaderAlignTopTrackBarChange(Sender: TObject);
begin
  UpdateHeaderAlignTopLabel;
end;

procedure TGlassDockOptionsFrame.SplitterWidthTrackBarChange(Sender: TObject);
begin
  UpdateSplitterWidthLabel;
end;

procedure TGlassDockOptionsFrame.OkClick(Sender: TObject);
begin
  if Settings <> nil then
    SaveToSettings(Settings);
  if Master <> nil then
    SaveToMaster;
end;

procedure TGlassDockOptionsFrame.ButtonSetDafaultClick(Sender: TObject);
begin
  LoadDefault;
end;

procedure TGlassDockOptionsFrame.DragThresholdTrackBarChange(Sender: TObject);
begin
  UpdateDragThresholdLabel;
end;

procedure TGlassDockOptionsFrame.SetMaster(const AValue: TGlassDockMaster);
begin
  if FMaster = AValue then
    exit;
  FMaster := AValue;
  if Master <> nil then
    LoadFromMaster;
end;

procedure TGlassDockOptionsFrame.SetFlags(AValue: TAnchorDockOptionsFlags);
begin
  if FFlags = AValue then
    Exit;
  FFlags := AValue;
  ApplyFlags;
end;

procedure TGlassDockOptionsFrame.SetSettings(AValue: TGlassDockSettings);
begin
  if FSettings = AValue then
    Exit;
  FSettings := AValue;
  if Settings <> nil then
    LoadFromSettings(Settings);
end;

procedure TGlassDockOptionsFrame.UpdateDragThresholdLabel;
begin
  DragThresholdLabel.Caption := adrsDragThreshold + ' (' + IntToStr(DragThresholdTrackBar.Position) + ')';
end;

procedure TGlassDockOptionsFrame.UpdateHeaderAlignTopLabel;
begin
  HeaderAlignTopLabel.Caption := adrsHeaderAlignTop + ' (' + IntToStr(HeaderAlignTopTrackBar.Position) + ')';
end;

procedure TGlassDockOptionsFrame.UpdateHeaderAlignLeftLabel;
begin
  HeaderAlignLeftLabel.Caption := adrsHeaderAlignLeft + ' (' + IntToStr(HeaderAlignLeftTrackBar.Position) + ')';
end;

procedure TGlassDockOptionsFrame.UpdateSplitterWidthLabel;
begin
  SplitterWidthLabel.Caption := adrsSplitterWidth + ' (' + IntToStr(SplitterWidthTrackBar.Position) + ')';
end;

procedure TGlassDockOptionsFrame.ApplyFlags;
begin
  ShowHeaderCheckBox.Visible := adofShow_ShowHeader in Flags;
end;

procedure TGlassDockOptionsFrame.SaveToMaster;
var
  CurSettings: TGlassDockSettings;
begin
  CurSettings := TGlassDockSettings.Create;
  try
    Master.SaveSettings(CurSettings);
    SaveToSettings(CurSettings);
    Master.LoadSettings(CurSettings);
  finally
    CurSettings.Free;
  end;
end;

procedure TGlassDockOptionsFrame.LoadFromMaster;
var
  CurSettings: TGlassDockSettings;
begin
  CurSettings := TGlassDockSettings.Create;
  try
    Master.SaveSettings(CurSettings);
    LoadFromSettings(CurSettings);
  finally
    CurSettings.Free;
  end;
end;

procedure TGlassDockOptionsFrame.LoadDefault;
var
  CurSettings: TGlassDockSettings;
begin
  CurSettings := TGlassDockSettings.Create;
  try
    CurSettings.ResetToDefault;
    LoadFromSettings(CurSettings);
  finally
    CurSettings.Free;
  end;
end;

procedure TGlassDockOptionsFrame.SaveToSettings(TheSettings: TGlassDockSettings);
begin

  if (HeaderColorComboBox.Selected = clNone) or (HeaderColorComboBox.Selected = clDefault) then
      TheSettings.HeaderColor := cnHeaderColorDefault else
      TheSettings.HeaderColor := HeaderColorComboBox.Selected;

  if (HeaderFontColorComboBox.Selected = clNone) or (HeaderFontColorComboBox.Selected = clDefault) then
      TheSettings.HeaderFontColor := cnHeaderFontColorDefault else
      TheSettings.HeaderFontColor := HeaderFontColorComboBox.Selected;

  if (SplitterColorComboBox.Selected = clNone) or (SplitterColorComboBox.Selected = clDefault) then
      TheSettings.SplitterColor := cnSplitterColorDefault else
      TheSettings.SplitterColor := SplitterColorComboBox.Selected;
                                                                    
  if (HeaderBorderColorComboBox.Selected = clDefault) then
      TheSettings.HeaderBorderColor := cnHeaderBorderColorDefault else
      TheSettings.HeaderBorderColor := HeaderBorderColorComboBox.Selected;

  TheSettings.HeaderIconsGroup := HeaderIconsGroupSpin.Value;
  TheSettings.DragTreshold := DragThresholdTrackBar.Position;
  TheSettings.HeaderAlignTop := HeaderAlignTopTrackBar.Position;
  TheSettings.HeaderAlignLeft := HeaderAlignLeftTrackBar.Position;
  TheSettings.SplitterWidth := SplitterWidthTrackBar.Position;
  TheSettings.ScaleOnResize := ScaleOnResizeCheckBox.Checked;
  TheSettings.ShowHeader := ShowHeaderCheckBox.Checked;
  TheSettings.ShowHeaderCaption := ShowHeaderCaptionCheckBox.Checked;
  TheSettings.ShowHeaderCloseButton := ShowHeaderCloseButtonCheckBox.Checked;
  TheSettings.HideHeaderCaptionFloatingControl := HideHeaderCaptionForFloatingCheckBox.Checked;
  TheSettings.DockSitesCanBeMinimized:=DockSitesCanBeMinimized.Checked;
  TheSettings.MultiLinePages:=DockSitesMultiLinePages.Checked;
end;

procedure TGlassDockOptionsFrame.LoadFromSettings(TheSettings: TGlassDockSettings);
begin

  HeaderColorComboBox.Selected := TheSettings.HeaderColor;
  HeaderBorderColorComboBox.Selected := TheSettings.HeaderBorderColor;  
  HeaderFontColorComboBox.Selected := TheSettings.HeaderFontColor;  
  HeaderIconsGroupSpin.Value:= TheSettings.HeaderIconsGroup;
  SplitterColorComboBox.Selected := TheSettings.SplitterColor;

  HeaderColor.Caption:=adrsHeaderColor;
  HeaderBorderColor.Caption:=adrsHeaderBorderColor; 
  HeaderFontColor.Caption:=adrsHeaderFontColor; 
  HeaderIconsGroup.Caption:=adrsHeaderIconsGroup;
  SplitterColor.Caption:=adrsSplitterColor;

  DragThresholdTrackBar.Hint := adrsAmountOfPixelTheMouseHasToDragBeforeDragStarts;
  DragThresholdTrackBar.Position := TheSettings.DragTreshold;
  UpdateDragThresholdLabel;

  HeaderAlignTopTrackBar.Hint := adrsMoveHeaderToTopWhenWidthHeight100HeaderAlignTop;
  HeaderAlignTopTrackBar.Position := TheSettings.HeaderAlignTop;
  UpdateHeaderAlignTopLabel;

  HeaderAlignLeftTrackBar.Hint := adrsMoveHeaderToLeftWhenWidthHeight100HeaderAlignLeft;
  HeaderAlignLeftTrackBar.Position := TheSettings.HeaderAlignLeft;
  UpdateHeaderAlignLeftLabel;

  SplitterWidthTrackBar.Hint := adrsSplitterThickness;
  SplitterWidthTrackBar.Position := TheSettings.SplitterWidth;
  UpdateSplitterWidthLabel;

  ScaleOnResizeCheckBox.Caption := adrsScaleOnResize;
  ScaleOnResizeCheckBox.Hint := adrsScaleSubSitesWhenASiteIsResized;
  ScaleOnResizeCheckBox.Checked := TheSettings.ScaleOnResize;

  ShowHeaderCheckBox.Caption := adrsShowHeaders;
  ShowHeaderCheckBox.Hint := adrsEachDockedWindowHasAHeaderThatAllowsDraggingHasACo;
  ShowHeaderCheckBox.Checked := TheSettings.ShowHeader;

  ShowHeaderCloseButtonCheckBox.Caption := adrsShowHeaderCloseButton;
  ShowHeaderCloseButtonCheckBox.Hint := adrsShowHeaderCloseButtonOfDockedControlsInTheHeader;
  ShowHeaderCloseButtonCheckBox.Checked := TheSettings.ShowHeaderCloseButton;

  ShowHeaderCaptionCheckBox.Caption := adrsShowHeaderCaptions;
  ShowHeaderCaptionCheckBox.Hint := adrsShowCaptionsOfDockedControlsInTheHeader;
  ShowHeaderCaptionCheckBox.Checked := TheSettings.ShowHeaderCaption;

  HideHeaderCaptionForFloatingCheckBox.Caption := adrsNoCaptionsForFloatingSites;
  HideHeaderCaptionForFloatingCheckBox.Hint := adrsHideHeaderCaptionsForSitesWithOnlyOneDockedControl;
  HideHeaderCaptionForFloatingCheckBox.Checked := TheSettings.HideHeaderCaptionFloatingControl;

  DockSitesCanBeMinimized.Checked:=TheSettings.DockSitesCanBeMinimized;
  DockSitesCanBeMinimized.Caption:=adrsAllowDockSitesToBeMinimized;
  DockSitesCanBeMinimized.Hint:=adrsAllowDockSitesToBeMinimized;

  DockSitesMultiLinePages.Checked:=TheSettings.MultiLinePages;
  DockSitesMultiLinePages.Caption:=adrsMultiLinePages;
  DockSitesMultiLinePages.Hint:=adrsMultiLinePagesHint;
end;

procedure TGlassDockOptionsFrame.HeaderIconsGroupSpinChange(Sender: TObject);
begin
   PanelForImages.Color:=HeaderColorComboBox.Selected;
   ImageClose.Picture.LoadFromResourceName(HInstance,VarBaseDockMaster.GetCloseBtnResName(HeaderIconsGroupSpin.Value));
   ImageMinimaze.Picture.LoadFromResourceName(HInstance,VarBaseDockMaster.GetMinimizeBtnResName(HeaderIconsGroupSpin.Value));
end;

end.
